#define STRICT
#include <windowsx.h>
#include <ddraw.h>
#include <d3d.h>
#include <mmsystem.h>
#include "DDInit.h"
#include "globals.h"
#include "graphics.h"

Graphics graphics;

//-----------------------------------------------------------------------------
// Name: checkanimate()
// Desc: Check to see whether to animate the tiles
//-----------------------------------------------------------------------------

bool DDInit::checkanimate(){
	static DWORD lastTickCount = 0;
	static DWORD thisTickCount;
	bool tempanim = false;
	DWORD delay1 = 250;
	thisTickCount = GetTickCount();

	// Move to next frame;
	if((thisTickCount - lastTickCount) > delay1){
		lastTickCount = thisTickCount;
		tempanim = true;
	}
	else{
		tempanim = false;
	}
	return tempanim;
}

//-----------------------------------------------------------------------------
// Name: DisplayFrame()
// Desc: Blts a the sprites to the back buffer, then it blts or flips the 
//       back buffer onto the primary buffer.
//-----------------------------------------------------------------------------

HRESULT DDInit::DisplayFrame(){
    HRESULT hr;
	
    // Fill the back buffer with black, ignoring errors until the flip
	DDisplay->Clear( 0 );
	
    // Blt all the sprites onto the back buffer using color keying,
    // ignoring errors until the last blt. Note that all of these sprites 
    // use the same DirectDraw surface.
	graphics.drawmenu();
	switch(currentmode){
	case EDITMAPS:
		graphics.drawmap();
		graphics.drawedittiles();
		break;
	default:
		break;
	}
    // We are in windowed mode so perform a blt from the backbuffer 
    // to the primary, returning any errors like DDERR_SURFACELOST
    if( FAILED( hr = DDisplay->Present() ) ){
        return hr;
	}

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: FreeDirectDraw()
// Desc: Release all the DirectDraw objects
//-----------------------------------------------------------------------------

void DDInit::FreeDirectDraw(){

	graphics.finish();
	SAFE_DELETE( DDisplay );
}

//-----------------------------------------------------------------------------
// Name: InitDirectDraw()
// Desc: Create the DirectDraw object, and init the surfaces
//-----------------------------------------------------------------------------

HRESULT DDInit::InitDirectDraw(HWND hWnd){
	HRESULT hr;
	DDisplay = new CDisplay();
	if( FAILED( hr = DDisplay->CreateWindowedDisplay(hWnd, WINDOW_WIDTH, WINDOW_HEIGHT ) ) ){
		return hr;
	}
	
	if( FAILED( hr = graphics.InitPictures() ) ){
		return hr;
	}
	graphics.initRect();
	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: ProcessNextFrame()
// Desc: Move the sprites, blt them to the back buffer, then 
//       flip or blt the back buffer to the primary buffer
//-----------------------------------------------------------------------------

HRESULT DDInit::ProcessNextFrame(HWND hWnd){
    HRESULT hr;
	
	bool tempanim1 = checkanimate();
	if(tempanim1 == true){
		graphics.changeanimrect();
	}
	if((animate == false)&&(tempanim1 == false)){
		return S_OK;
	}
	
    // Check the cooperative level before rendering
    if( FAILED( hr = DDisplay->GetDirectDraw()->TestCooperativeLevel() ) ){
        switch( hr )
        {
		case DDERR_EXCLUSIVEMODEALREADYSET:
			// Do nothing because some other app has exclusive mode
			Sleep(10);
			return S_OK;
			
		case DDERR_WRONGMODE:
			// The display mode changed on us. Update the
			// DirectDraw surfaces accordingly
			FreeDirectDraw();
			return InitDirectDraw(hWnd);
        }
        return hr;
    }
	
    // Display the sprites on the screen
    if( FAILED( hr = DisplayFrame() ) ){
        if( hr != DDERR_SURFACELOST )
            return hr;
		
        // The surfaces were lost so restore them 
        RestoreSurfaces();
    }
	
	animate = false;

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RestoreSurfaces()
// Desc: Restore all the surfaces, and redraw the sprite surfaces.
//-----------------------------------------------------------------------------

HRESULT DDInit::RestoreSurfaces(){
    HRESULT hr;
	
    if( FAILED( hr = DDisplay->GetDirectDraw()->RestoreAllSurfaces() ) ){
        return hr;
	}
	
	return graphics.RestoreSurfaces();
}

HRESULT createmapcoordinates(){
	return graphics.createmapcoordinates(currentxpos, currentypos);
}
